rm(list = ls() )
library(dynlm)
library(reshape2)
library(ggplot2)
library(dplyr)
library(tidyr)
library(lubridate)
library(cowplot)
library(lattice)
library(gridExtra)

setwd("~/bHP_empirical")
source("RScripts/BoostedHP.R")


# import the data 
df = read.csv("quarterly.csv")
dates <- df[,1]
dates <- as.Date.character(dates,format = "%m/%d/%Y")
names(df)[1] <- "DATE"
df[,1] <- dates

x_raw <- df$GDPC1
x = x_raw[!is.na(x_raw)]



hp = BoostedHP(x, lambda=1600, iter = F)
cycle_HP = hp$cycle
tmp = x_raw
tmp[!is.na(tmp)] = cycle_HP
cycle_HP = tmp 



twohp = BoostedHP(cycle_HP[!is.na(cycle_HP)], lambda=1600, iter = F)
cycle_2HP = twohp$cycle
tmp = x_raw
tmp[!is.na(tmp)] = cycle_2HP
cycle_2HP = tmp 


bhp.bic = BoostedHP(x, lambda=1600, iter = TRUE, stopping =  "BIC", Max_Iter = 200)
cycle_BIC = bhp.bic$cycle
tmp = x_raw
tmp[!is.na(tmp)] = cycle_BIC
cycle_BIC = tmp 



reg = dynlm( as.ts(x)  ~ L(as.ts(x), c(8:11) ) )
d4 = c( rep(NA,11), predict(reg) )
cycle_AR4 <- x - d4 
tmp = x_raw
tmp[!is.na(tmp)] = cycle_AR4
cycle_AR4 = as.numeric(tmp)  


dcycle_GDP <- df[,c("DATE","GDPC1")]
names(dcycle_GDP) <- c("DATE","GDP")
dcycle_GDP$BIC <- cycle_BIC
dcycle_GDP$AR4 <- cycle_AR4
dcycle_GDP$HP <- cycle_HP
dcycle_GDP$twoHP <- cycle_2HP
 

graph_fun <- function(series_df, recess_df)  {
  ggp = ggplot(series_df, aes(x = DATE, y = GDP )) + geom_point(alpha = 0.5 )
  
  if (colnames(series_df[2]) == "HP" ){
    ggp = ggp + geom_line(aes(x = DATE, y =  GDP - HP ), color = "red", size = 1, alpha=0.9 )
  } else if ( (colnames(series_df[2]) == "twoHP" ) ){
    ggp = ggp + geom_line(aes(x = DATE, y =  GDP - twoHP ), color = "darkgreen", size = 1, alpha=0.9 )
  } else if ( (colnames(series_df[2]) == "BIC" ) ){
    ggp = ggp + geom_line(aes(x = DATE, y =  GDP - BIC ), color = "blue", size = 1, alpha=0.9 )
  } else if ((colnames(series_df[2]) == "AR4" )){
    ggp = ggp + geom_line(aes(x = DATE, y =  GDP - AR4 ), color = "purple", size = 1, alpha=0.9 )
  }
  
  ggp = ggp + xlab("")+ ylab("") + 
    scale_x_date(date_breaks = "5 years", date_minor_breaks = "1 year", 
                 date_labels = "%Y", limits = as.Date(c("1959-01-01","2023-07-01")))
  ggp = ggp + annotate("rect",xmin = as.Date(recession_1959.df$start, "%Y-%m-%d"),
                       xmax= as.Date(recession_1959.df$end, "%Y-%m-%d"),
                       ymin=-Inf,ymax=Inf, alpha=0.2, fill="black") 
  
  # browser()
  if ( recess_df[1,1] < "1990-01-01" ) {
    ggp = ggp + scale_x_date(date_breaks = "5 year", date_minor_breaks = "1 year", date_labels= "%Y")
  } else {
    ggp = ggp + scale_x_date(date_breaks = "1 year", date_minor_breaks = "1 year", date_labels= "%Y" )
  }
  ggp = ggp + theme_bw()+ theme(legend.position = "none",
                                axis.text=element_text(size=16))
  
  return(ggp)
}


## recession dates
load("date_recession.Rdata")
df.temp = recession.df[1,]
df.temp[1,] <- c("2020-03-01","2020-04-01")
recession.df <- rbind(recession.df,df.temp)
month(recession.df$start) <- month(recession.df$start) - 1
recession.df[26,2] <- recession.df[26,2] - 2
month(recession.df$end) <- month(recession.df$end) + 1
recession.df$end <- recession.df$end - 1
recession_1959.df = recession.df[recession.df$start>=min(df$DATE),]
recession_1959.df[1,] <- c("1960-06-01","1961-03-01")
recession_1959.df[2,] <- c("1969-12-01","1970-12-01")
recession_1959.df[3,] <- c("1973-12-01","1975-03-01")
recession_1959.df[4,] <- c("1980-03-01","1980-09-01")
recession_1959.df[5,] <- c("1981-09-01","1982-12-01")
recession_1959.df[6,] <- c("1990-09-01","1991-03-01")
recession_1959.df[7,] <- c("2001-03-01","2001-12-01")
recession_1959.df[8,] <- c("2007-12-01","2009-06-01")
recession_1959.df[9,] <- c("2019-12-01","2020-06-01")
theme1 = theme_bw( # base_family = "Times New Roman"
) +  theme(legend.title.align = 0.5, 
           plot.title = element_text(hjust = 0.5),
           legend.position="bottom", 
           text = element_text(size = 22),
           legend.title=element_text(size=22),
           panel.grid.major = element_blank(),
           panel.grid.minor = element_blank(),
           axis.text = element_text(size=22),
           axis.title = element_text(size=22),
           legend.text=element_text(size=22)
) 


########### Trend 

glist <- list()

ggp_GDP = graph_fun(dcycle_GDP[, c("DATE","HP","GDP")], recession_1959.df )

ggp_GDP = ggp_GDP + theme1  + 
  theme(axis.text.x = element_blank(), 
        axis.ticks.x = element_blank(), 
        axis.title.x = element_blank()) + 
  #scale_y_continuous(limits = c(-6,6),breaks = seq(-6,6,3))+
  theme(plot.background = element_rect(color = "white",
                                       size = 1),
        plot.margin = margin(t = 10,   
                             r = 20,   
                             b = -10,   
                             l = 5)) + xlab(" ") + ylab("HP")

glist[[1]] <- ggp_GDP 

# 
# ggp_GDP = graph_fun(dcycle_GDP[, c("DATE","twoHP","GDP")], recession_1959.df )
# 
# ggp_GDP = ggp_GDP + theme1  + 
#   theme(axis.text.x = element_blank(), 
#         axis.ticks.x = element_blank(), 
#         axis.title.x = element_blank()) + 
#   #scale_y_continuous(limits = c(-6,6),breaks = seq(-6,6,3))+
#   theme(plot.background = element_rect(color = "white",
#                                        size = 1),
#         plot.margin = margin(t = -20,   
#                              r = 20,   
#                              b = -5,   
#                              l = 5)) + xlab(" ") + ylab("2HP")
# 
# glist[[2]] <- ggp_GDP 

ggp_GDP = graph_fun(dcycle_GDP[, c("DATE","BIC","GDP")], recession_1959.df )
ggp_GDP = ggp_GDP + theme1  + 
  theme(axis.text.x = element_blank(), 
        axis.ticks.x = element_blank(), 
        axis.title.x = element_blank()) + 
  #scale_y_continuous(limits = c(-6,6),breaks = seq(-6,6,3))+
  theme(plot.background = element_rect(color = "white",
                                       size = 1),
        plot.margin = margin(t = -20,   
                             r = 20,   
                             b = -5,   
                             l = 5)) + xlab(" ") + ylab("bHP")

glist[[2]] <- ggp_GDP 


ggp_GDP = graph_fun(dcycle_GDP[, c("DATE","AR4","GDP")], recession_1959.df )

ggp_GDP = ggp_GDP + theme1  + 
  #scale_y_continuous(limits = c(-6,6),breaks = seq(-6,6,3))+
  theme(plot.background = element_rect(color = "white",
                                       size = 1),
        plot.margin = margin(t = -20,   
                             r = 20,   
                             b = -5,   
                             l = 5)) + xlab(" ") + ylab("HRF")

glist[[3]] <- ggp_GDP 
p.grid <- plot_grid(
  plotlist = glist, ncol = 1, align = "hv"
)
cairo_pdf("graphs/GDP_trend.pdf", width = 15, height = 16, family = "Times",
          fallback_resolution = 1000) 
print(p.grid )
dev.off()


##################### cycle 



gp1 = ggplot(dcycle_GDP,aes(x = DATE, y = HP )) + 
  geom_line(aes(x = DATE, y = HP ), color = "red", size = 1, alpha=0.9) + 
  theme1  + 
  theme(axis.text.x = element_blank(), 
        axis.ticks.x = element_blank(), 
        axis.title.x = element_blank()) + 
  scale_x_date(date_breaks = "5 years", date_minor_breaks = "1 year", 
               date_labels = "%Y", limits = as.Date(c("1959-01-01","2023-07-01"))) + 
  annotate("rect",xmin = as.Date(recession_1959.df$start, "%Y-%m-%d"),
           xmax= as.Date(recession_1959.df$end, "%Y-%m-%d"),
           ymin=-Inf,ymax=Inf, alpha=0.2, fill="black")  + 
  scale_y_continuous(limits = c(-2150,1250),breaks = seq(-2100,1200,600))+
  theme(plot.background = element_rect(color = "white",
                                       size = 1),
        plot.margin = margin(t = 10,   
                             r = 20,   
                             b = -10,   
                             l = 5)) 

# 
# gp2 = ggplot(dcycle_GDP, aes(x = DATE, y = twoHP )) + 
#   geom_line(aes(x = DATE, y = twoHP ), color = "darkgreen", size = 1, alpha=0.9) + 
#   theme1  + ylab("2HP") + 
#   theme(axis.text.x = element_blank(), 
#         axis.ticks.x = element_blank(), 
#         axis.title.x = element_blank()) + 
#   scale_x_date(date_breaks = "5 years", date_minor_breaks = "1 year", 
#                date_labels = "%Y", limits = as.Date(c("1959-01-01","2022-03-01"))) + 
#   annotate("rect",xmin = as.Date(recession_1959.df$start, "%Y-%m-%d"),
#            xmax= as.Date(recession_1959.df$end, "%Y-%m-%d"),
#            ymin=-Inf,ymax=Inf, alpha=0.2, fill="black") + 
#   scale_y_continuous(limits = c(-1900,950),breaks = seq(-1800,900,300))+
#   theme(plot.background = element_rect(color = "white",
#                                        size = 1),
#         plot.margin = margin(t = -1000,   
#                              r = 20,   
#                              b = -1000,   
#                              l = 5))  

gp2 = ggplot(dcycle_GDP, aes(x = DATE, y = BIC )) + 
  geom_line(aes(x = DATE, y = BIC ), color = "blue", size = 1, alpha=0.9) + 
  theme1 + ylab("bHP") + 
  theme(axis.text.x = element_blank(), 
        axis.ticks.x = element_blank(), 
        axis.title.x = element_blank()) + 
  scale_x_date(date_breaks = "5 years", date_minor_breaks = "1 year", 
               date_labels = "%Y", limits = as.Date(c("1959-01-01","2023-07-01"))) + 
  annotate("rect",xmin = as.Date(recession_1959.df$start, "%Y-%m-%d"),
           xmax= as.Date(recession_1959.df$end, "%Y-%m-%d"),
           ymin=-Inf,ymax=Inf, alpha=0.2, fill="black") + 
  scale_y_continuous(limits = c(-2150,1250),breaks = seq(-2100,1200,600))+
  theme(plot.background = element_rect(color = "white",
                                       size = 1),
        plot.margin = margin(t = -1000,   
                             r = 20,   
                             b = -1000,   
                             l = 5)) 


gp3 = ggplot(dcycle_GDP, aes(x = DATE, y = AR4 )) + 
  geom_line(aes(x = DATE, y = AR4 ), color = "purple", size = 1, alpha=0.9) + 
  theme1 +  
  ylab("HRF") + 
  scale_x_date(date_breaks = "5 years", date_minor_breaks = "1 year", 
               date_labels = "%Y", limits = as.Date(c("1959-01-01","2023-07-01"))) + 
  annotate("rect",xmin = as.Date(recession_1959.df$start, "%Y-%m-%d"),
           xmax= as.Date(recession_1959.df$end, "%Y-%m-%d"),
           ymin=-Inf,ymax=Inf, alpha=0.2, fill="black") + 
  scale_y_continuous(limits = c(-2150,1250),breaks = seq(-2100,1200,600))+
  theme(plot.background = element_rect(color = "white",
                                       size = 1),
        plot.margin = margin(t = -20,   
                             r = 20,   
                             b = -5,   
                             l = 5)) + xlab(" ")

glist <- list()
glist[[1]] <- gp1 
glist[[2]] <- gp2 
glist[[3]] <- gp3 
# glist[[4]] <- gp4 

p.grid <- plot_grid(
  plotlist = glist, ncol = 1, align = "hv"
)
cairo_pdf("graphs/GDP_cycle.pdf", width = 15, height = 16, family = "Times",
          fallback_resolution = 1000)
print(p.grid )
dev.off()
