############### Quarterly Data 

rm(list = ls() )
library(dynlm)
library(reshape2)
setwd("~/bHP_empirical")
source("RScripts/BoostedHP.R")

# import the data 
df = read.csv("quarterly.csv")
dates <- df[,1]
dates <- as.Date.character(dates,format = "%m/%d/%Y")
names(df)[1] <- "DATE"
df[,1] <- dates



## initialization 
var_names <- colnames(df)
df_HP = df 
df_2HP = df 
df_BIC = df
df_AR4 = df  

for (name in var_names[2:length(var_names)]){
  
  x_raw = df[,name]
  x = x_raw[!is.na(x_raw)]
  
  hp = BoostedHP(x, lambda=1600, iter = F)
  cycle_HP = hp$cycle
  tmp = x_raw
  tmp[!is.na(tmp)] = cycle_HP
  cycle_HP = tmp 
  
  twohp = BoostedHP(cycle_HP[!is.na(cycle_HP)], lambda=1600, iter = F)
  cycle_2HP = twohp$cycle
  tmp = x_raw
  tmp[!is.na(tmp)] = cycle_2HP
  cycle_2HP = tmp 
  
  
  bhp.bic = BoostedHP(x, lambda=1600, iter = TRUE, stopping =  "BIC", Max_Iter = 200)
  cycle_BIC = bhp.bic$cycle
  tmp = x_raw
  tmp[!is.na(tmp)] = cycle_BIC
  cycle_BIC = tmp 
  
  
  
  reg = dynlm( as.ts(x)  ~ L(as.ts(x), c(8:11) ) )
  d4 = c( rep(NA,11), predict(reg) )
  cycle_AR4 <- x - d4 
  tmp = x_raw
  tmp[!is.na(tmp)] = cycle_AR4
  cycle_AR4 = as.numeric(tmp) 
  
  df_HP[,name] <- cycle_HP 
  df_2HP[,name] <- cycle_2HP 
  df_BIC[,name] <- cycle_BIC 
  df_AR4[,name] <- cycle_AR4
}
saveRDS(df_HP,file="RDS/cycle_HP.RDS")
saveRDS(df_2HP,file="RDS/cycle_2HP.RDS")
saveRDS(df_BIC,file="RDS/cycle_BIC.RDS")
saveRDS(df_AR4,file="RDS/cycle_AR4.RDS")


############## Monthly Data 

rm(list = ls() )
library(dynlm)
library(reshape2)
setwd("~/bHP_empirical")
source("RScripts/BoostedHP.R")

# import the data 
df = read.csv("monthly.csv")
dates <- df[,1]
dates <- as.Date.character(dates,format = "%m/%d/%Y")
names(df)[1] <- "DATE"
df[,1] <- dates



## initialization 
var_names <- colnames(df)
df_HP = df 
df_2HP = df 
df_BIC = df
df_AR12 = df 


for (name in var_names[2:length(var_names)]){
  
  x_raw = df[,name]
  x = x_raw[!is.na(x_raw)]
  
  hp = BoostedHP(x, lambda=129600, iter = F)
  cycle_HP = hp$cycle
  tmp = x_raw
  tmp[!is.na(tmp)] = cycle_HP
  cycle_HP = tmp 
  
  twohp = BoostedHP(cycle_HP[!is.na(cycle_HP)], lambda=129600, iter = F)
  cycle_2HP = twohp$cycle
  tmp = x_raw
  tmp[!is.na(tmp)] = cycle_2HP
  cycle_2HP = tmp 
  
  bhp.bic = BoostedHP(x, lambda=129600, iter = TRUE, stopping =  "BIC", Max_Iter = 200)
  cycle_BIC = bhp.bic$cycle
  tmp = x_raw
  tmp[!is.na(tmp)] = cycle_BIC
  cycle_BIC = tmp 
  
  
  
  reg = dynlm( as.ts(x)  ~ L(as.ts(x), c(24:35) ) )
  d12 = c( rep(NA,35), predict(reg) )
  cycle_AR12 <- x - d12
  tmp = x_raw
  tmp[!is.na(tmp)] = cycle_AR12
  cycle_AR12 = as.numeric(tmp) 
  
  df_HP[,name] <- cycle_HP 
  df_2HP[,name] <- cycle_2HP 
  df_BIC[,name] <- cycle_BIC 
  df_AR12[,name] <- cycle_AR12
}
saveRDS(df_HP,file="RDS/cycle_HP_month.RDS")
saveRDS(df_2HP,file="RDS/cycle_2HP_month.RDS")
saveRDS(df_BIC,file="RDS/cycle_BIC_month.RDS")
saveRDS(df_AR12,file="RDS/cycle_AR12_month.RDS")
