rm(list = ls() )
library(dynlm)
library(reshape2)
library(ggplot2)
library(dplyr)
library(tidyr)
library(lubridate)
library(cowplot)
library(lattice)
library(gridExtra)

setwd("~/bHP_empirical")
source("RScripts/BoostedHP.R")

# import the data 
df = read.csv("quarterly.csv")
dates <- df[,1]
dates <- as.Date.character(dates,format = "%m/%d/%Y")
names(df)[1] <- "DATE"
df[,1] <- dates

x_raw <- df$GDPC1
x = x_raw[!is.na(x_raw)]

# 
# 
# hp = BoostedHP(x, lambda=1600, iter = F)
# cycle_HP = hp$cycle
# tmp = x_raw
# tmp[!is.na(tmp)] = cycle_HP
# cycle_HP = tmp 
# 
# 
# twohp = BoostedHP(cycle_HP[!is.na(cycle_HP)], lambda=1600, iter = F)
# cycle_2HP = twohp$cycle
# tmp = x_raw
# tmp[!is.na(tmp)] = cycle_2HP
# cycle_2HP = tmp 

 

reg = dynlm( as.ts(x)  ~ L(as.ts(x), c(1:4) ) )
d4 = c( rep(NA,4), predict(reg) )
cycle_AR4 <- x - d4 
tmp = x_raw
tmp[!is.na(tmp)] = cycle_AR4
cycle_AR01 = as.numeric(tmp)  

reg = dynlm( as.ts(x)  ~ L(as.ts(x), c(4:7) ) )
d4 = c( rep(NA,7), predict(reg) )
cycle_AR4 <- x - d4 
tmp = x_raw
tmp[!is.na(tmp)] = cycle_AR4
cycle_AR04 = as.numeric(tmp)  

reg = dynlm( as.ts(x)  ~ L(as.ts(x), c(8:11) ) )
d4 = c( rep(NA,11), predict(reg) )
cycle_AR4 <- x - d4 
tmp = x_raw
tmp[!is.na(tmp)] = cycle_AR4
cycle_AR08 = as.numeric(tmp)  

reg = dynlm( as.ts(x)  ~ L(as.ts(x), c(12:15) ) )
d4 = c( rep(NA,15), predict(reg) )
cycle_AR4 <- x - d4 
tmp = x_raw
tmp[!is.na(tmp)] = cycle_AR4
cycle_AR12 = as.numeric(tmp)  


dcycle_GDP <- df[,c("DATE","GDPC1")]
names(dcycle_GDP) <- c("DATE","GDP")
dcycle_GDP$AR1 <- cycle_AR01
dcycle_GDP$AR4 <- cycle_AR04
dcycle_GDP$AR8 <- cycle_AR08
dcycle_GDP$AR12 <- cycle_AR12


graph_fun <- function(series_df, recess_df)  {
  ggp = ggplot(series_df, aes(x = DATE, y = GDP )) + geom_point(alpha = 0.5 )
  
  if (colnames(series_df[2]) == "HP" ){
    ggp = ggp + geom_line(aes(x = DATE, y =  GDP - HP ), color = "red", size = 1, alpha=0.9 )
  } else if ( (colnames(series_df[2]) == "twoHP" ) ){
    ggp = ggp + geom_line(aes(x = DATE, y =  GDP - twoHP ), color = "darkforestgreen", size = 1, alpha=0.9 )
  } else if ( (colnames(series_df[2]) == "BIC" ) ){
    ggp = ggp + geom_line(aes(x = DATE, y =  GDP - BIC ), color = "blue", size = 1, alpha=0.9 )
  } else if ((colnames(series_df[2]) == "AR4" )){
    ggp = ggp + geom_line(aes(x = DATE, y =  GDP - AR4 ), color = "purple", size = 1, alpha=0.9 )
  }
  
  
  ggp = ggp + annotate("rect",xmin = as.Date(recession_1959.df$start, "%Y-%m-%d"),
                       xmax= as.Date(recession_1959.df$end, "%Y-%m-%d"),
                       ymin=-Inf,ymax=Inf, alpha=0.2, fill="black")  +
    geom_vline(xintercept = as.Date(recession_8q.df, "%Y-%m-%d"), linetype = "dashed",
               color = "black", alpha = 0.5)
  
  # browser()
  if ( recess_df[1,1] < "1990-01-01" ) {
    ggp = ggp + scale_x_date(date_breaks = "5 year", date_minor_breaks = "1 year", date_labels= "%Y")
  } else {
    ggp = ggp + scale_x_date(date_breaks = "2 year", date_minor_breaks = "1 year", date_labels= "%Y" )
  }
  ggp = ggp + theme_bw()+ theme(legend.position = "none",
                                axis.text=element_text(size=16))+ 
    scale_y_continuous(limits = c(12000,20500),breaks = seq(12000,20000,2000))
  
  return(ggp)
}


## recession dates
load("date_recession.Rdata")
df.temp = recession.df[1,]
df.temp[1,] <- c("2020-03-01","2020-04-01")
recession.df <- rbind(recession.df,df.temp)
month(recession.df$start) <- month(recession.df$start) - 1
recession.df[26,2] <- recession.df[26,2] - 2
month(recession.df$end) <- month(recession.df$end) + 1
recession.df$end <- recession.df$end - 1
recession_1959.df = recession.df[recession.df$start>=min(df$DATE),]

recession_1959.df[1,] <- c("1960-06-01","1961-03-01")
recession_1959.df[2,] <- c("1969-12-01","1970-12-01")
recession_1959.df[3,] <- c("1973-12-01","1975-03-01")
recession_1959.df[4,] <- c("1980-03-01","1980-09-01")
recession_1959.df[5,] <- c("1981-09-01","1982-12-01")
recession_1959.df[6,] <- c("1990-09-01","1991-03-01")
recession_1959.df[7,] <- c("2001-03-01","2001-12-01")
recession_1959.df[8,] <- c("2007-12-01","2009-06-01")
recession_1959.df[9,] <- c("2019-12-01","2020-06-01")


recession_1q.df <- recession_1959.df$start
recession_1q.df[1]<-"1961-06-01"
recession_1q.df[2]<-"1971-03-01"
recession_1q.df[3]<-"1975-06-01"
recession_1q.df[4]<-"1980-12-01"
recession_1q.df[5]<-"1983-03-01"
recession_1q.df[6]<-"1991-06-01"
recession_1q.df[7]<-"2002-03-01"
recession_1q.df[8]<-"2009-09-01"
recession_1q.df[9]<-"2020-09-01"



recession_4q.df <- recession_1959.df$start
recession_4q.df[1]<-"1962-03-01"
recession_4q.df[2]<-"1971-12-01"
recession_4q.df[3]<-"1976-03-01"
recession_4q.df[4]<-"1981-09-01"
recession_4q.df[5]<-"1983-12-01"
recession_4q.df[6]<-"1992-03-01"
recession_4q.df[7]<-"2002-12-01"
recession_4q.df[8]<-"2010-06-01"
recession_4q.df[9]<-"2021-06-01"







recession_8q.df <- recession_1959.df$start
recession_8q.df[1]<-"1963-03-01"
recession_8q.df[2]<-"1972-12-01"
recession_8q.df[3]<-"1977-03-01"
recession_8q.df[4]<-"1982-09-01"
recession_8q.df[5]<-"1984-12-01"
recession_8q.df[6]<-"1993-03-01"
recession_8q.df[7]<-"2003-12-01"
recession_8q.df[8]<-"2011-06-01"
recession_8q.df[9]<-"2022-06-01"



recession_12q.df <- recession_1959.df$start
recession_12q.df[1]<-"1964-03-01"
recession_12q.df[2]<-"1973-12-01"
recession_12q.df[3]<-"1978-03-01"
recession_12q.df[4]<-"1983-09-01"
recession_12q.df[5]<-"1985-12-01"
recession_12q.df[6]<-"1994-03-01"
recession_12q.df[7]<-"2004-12-01"
recession_12q.df[8]<-"2012-06-01"
recession_12q.df[9]<-"2023-06-01"






theme1 = theme_bw( # base_family = "Times New Roman"
) +  theme(legend.title.align = 0.5,
           plot.title = element_text(hjust = 0.5),
           legend.position= "bottom",
           text = element_text(size = 22),
           legend.title=element_text(size=22),
           panel.grid.major = element_blank(),
           panel.grid.minor = element_blank(),
           axis.text = element_text(size=22),
           axis.title = element_text(size=22),
           legend.text=element_text(size=22)
)

##################### cycle 
shapes <- c("h=1" = 15, "h=4" = 17, "h=8" = 16, "h=12" = 18)
colors <- c("h=1" = "forestgreen", "h=4" = "sienna", "h=8" = "purple", "h=12" = "gold2")

gp1 = ggplot(dcycle_GDP,aes(x = DATE, y = AR1 )) + 
  geom_point(aes(x = DATE, y = AR1, shape = "h=1", color = "h=1"),  size = 2.5, alpha=0.45) + 
  geom_line(aes(x = DATE, y = AR1), color = "forestgreen", size = .8, alpha=0.3, linetype = "solid") +
  geom_point(aes(x = DATE, y = AR4, shape = "h=4", color = "h=4"),   size = 2.5, alpha=0.45) + 
  geom_line(aes(x = DATE, y = AR4), color = "sienna", size = .8, alpha=0.45, linetype = "solid") +
  geom_point(aes(x = DATE, y = AR8, shape = "h=8", color = "h=8"), size = 2.5, alpha=0.9) + 
  geom_line(aes(x = DATE, y = AR8), color = "purple", size = .8, alpha=0.9, linetype = "solid") +
  geom_point(aes(x = DATE, y = AR12, shape = "h=12", color = "h=12"), size = 2.5, alpha=0.45, ) + 
  geom_line(aes(x = DATE, y = AR12), color = "gold2", size = .8, alpha=0.6, linetype = "solid") +
  theme1  +
  theme(axis.title.x = element_blank(),
        axis.title.y = element_blank()) +
  # axis.text.x = element_blank(),
  # axis.ticks.x = element_blank(),
  scale_x_date(date_breaks = "2 years", date_minor_breaks = "1 year",
               date_labels = "%Y", limits = as.Date(c("2000-01-01","2023-07-01"))) +
  annotate("rect",xmin = as.Date(recession_1959.df$start, "%Y-%m-%d"),
           xmax= as.Date(recession_1959.df$end, "%Y-%m-%d"),
           ymin=-Inf,ymax=Inf, alpha=0.2, fill="black") +
  geom_vline(xintercept = as.Date(recession_1q.df, "%Y-%m-%d"), linetype = "dashed",
             color = "forestgreen", alpha = 0.5)+
  geom_vline(xintercept = as.Date(recession_4q.df, "%Y-%m-%d"), linetype = "dashed",
             color = "sienna", alpha = 0.5)+
  geom_vline(xintercept = as.Date(recession_8q.df, "%Y-%m-%d"), linetype = "dashed",
             color = "purple", alpha = 0.5)+
  geom_vline(xintercept = as.Date(recession_12q.df, "%Y-%m-%d"), linetype = "dashed",
             color = "gold2", alpha = 0.5) +
  geom_hline(yintercept = 0, linetype = "dashed",
             color = "black", alpha = 0.5)+
  scale_y_continuous(limits = c(-2150,1500),breaks = seq(-2100,1500,600))+
  theme(plot.background = element_rect(color = "white",
                                       size = 1),
        plot.margin = margin(t = 10,
                             r = 20,
                             b = 20,
                             l = 5))  + 
  scale_shape_manual(name = "", values = shapes)+ 
  scale_color_manual(name = "", values = colors)
cairo_pdf("graphs/conclusion.pdf", width = 15, height = 8, family = "Times",
          fallback_resolution = 1000)
print(gp1)
dev.off()
# 
# ############################################
# 
# gp1 = ggplot(dcycle_GDP,aes(x = DATE, y = AR1 )) + 
#   geom_point(aes(x = DATE, y = AR1 ), color = "purple", size = 2.5, alpha=0.3, shape = 15) + 
#   geom_line(aes(x = DATE, y = AR1), color = "purple", size = .8, alpha=0.3, linetype = "dotted") +
#   geom_point(aes(x = DATE, y = AR4 ), color = "purple", size = 2.5, alpha=0.45, shape = 17) + 
#   geom_line(aes(x = DATE, y = AR4), color = "purple", size = .8, alpha=0.45, linetype = "dotdash") +
#   geom_point(aes(x = DATE, y = AR8 ), color = "purple", size = 2.5, alpha=0.9, shape = 16) + 
#   geom_line(aes(x = DATE, y = AR8), color = "purple", size = .8, alpha=0.9, linetype = "solid") +
#   geom_point(aes(x = DATE, y = AR12 ), color = "purple", size = 2.5, alpha=0.6, , shape = 18) + 
#   geom_line(aes(x = DATE, y = AR12), color = "purple", size = .8, alpha=0.6, linetype = "dashed") +
#   theme1  +
#   theme(axis.title.x = element_blank(),
#         axis.title.y = element_blank()) +
#   # axis.text.x = element_blank(),
#   # axis.ticks.x = element_blank(),
#   scale_x_date(date_breaks = "2 years", date_minor_breaks = "1 year",
#                date_labels = "%Y", limits = as.Date(c("2000-01-01","2023-07-01"))) +
#   annotate("rect",xmin = as.Date(recession_1959.df$start, "%Y-%m-%d"),
#            xmax= as.Date(recession_1959.df$end, "%Y-%m-%d"),
#            ymin=-Inf,ymax=Inf, alpha=0.2, fill="black") +
#   geom_vline(xintercept = as.Date(recession_8q.df, "%Y-%m-%d"), linetype = "dashed",
#              color = "black", alpha = 0.5) +
#   geom_hline(yintercept = 0, linetype = "dashed",
#              color = "black", alpha = 0.5)+
#   scale_y_continuous(limits = c(-2150,1500),breaks = seq(-2100,1500,600))+
#   theme(plot.background = element_rect(color = "white",
#                                        size = 1),
#         plot.margin = margin(t = 10,
#                              r = 20,
#                              b = 20,
#                              l = 5)) +
#   scale_color_manual(labels=c("h=1","h=4","h=8","h=12") , values = c("purple", "purple", "purple", "purple"))
# cairo_pdf("graphs/conclusion.pdf", width = 15, height = 8, family = "Times",
#           fallback_resolution = 1000)
# print(gp1)
# dev.off()


