### plot 
rm(list = ls())

library(ggplot2)
library(reshape2)
library(dplyr)
library(tidyr)
library(lubridate)
library(cowplot)
library(lattice)
library(gridExtra)

setwd("~/bHP_empirical")
df_HP <- readRDS("RDS/cycle_HP.RDS")
df_2HP <- readRDS("RDS/cycle_2HP.RDS")
df_BIC <- readRDS("RDS/cycle_BIC.RDS")
df_AR4 <- readRDS("RDS/cycle_AR4.RDS")


dates <- df_BIC[,1]
var_names <- colnames(df_BIC)


# Variable flipping  
negative_ind = c("UNRATE"  ,      "UNRATESTx"  ,   "UNRATELTx"   ,  "LNS14000012"  , "LNS14000025"  , "LNS14000026"  ,
 "UEMPLT5"   ,    "UEMP5TO14"   ,  "UEMP15T26" ,    "UEMP27OV"  ,    "LNS13023621"  , "LNS13023557"  ,
  "LNS13023705"  , "LNS13023569"  , "UEMPMEAN" ,     "BOGMBASEREALx", "M1REAL"   ,     "M2REAL"   )
# , "EXUSUKx"    ,   "EXUSEU" 
# c(59:72,195,158:160,182,183,185) + 1
df_HP[,negative_ind] = -df_HP[,negative_ind]
df_2HP[,negative_ind] = -df_2HP[,negative_ind]
df_BIC[,negative_ind] = -df_BIC[,negative_ind]
df_AR4[,negative_ind] = -df_AR4[,negative_ind]


## Normalization 
for (i in 2:length(var_names)){
  
  df_HP[,i] <- df_HP[,i]/sd(df_HP[,i], na.rm = T)
  
  df_2HP[,i] <- df_2HP[,i]/sd(df_2HP[,i], na.rm = T)
  
  df_BIC[,i] <- df_BIC[,i]/sd(df_BIC[,i], na.rm = T)
  
  df_AR4[,i] <- df_AR4[,i]/sd(df_AR4[,i], na.rm = T)
}



# dHP_mean <- df_HP["DATE"]
# dHP_mean$mean <- rowMeans(df_HP[,2:length(var_names)], na.rm = T)
# dHP_mean_varyingLambda <- df_HP["DATE"]
# dHP_mean_varyingLambda$mean <- rowMeans(df_HP_varyingLambda[,2:length(var_names)], na.rm = T)
# 
# dBIC_mean <- df_HP["DATE"]
# dBIC_mean$mean <- rowMeans(df_BIC[,2:length(var_names)], na.rm = T)
# dBIC_mean_varyingLambda <- df_HP["DATE"]
# dBIC_mean_varyingLambda$mean <- rowMeans(df_BIC_varyingLambda[,2:length(var_names)], na.rm = T)
# 
# dAR4_mean <- df_AR4["DATE"]
# dAR4_mean$mean <- rowMeans(df_AR4[,2:length(var_names)], na.rm = T)
# dAR4_mean_varyingLambda <- df_AR4["DATE"]
# dAR4_mean_varyingLambda$mean <- rowMeans(df_AR4_varyingLambda[,2:length(var_names)], na.rm = T)

# mean((dHP_mean_split$mean - dHP_mean$mean)^2,na.rm = T)
# mean((dBIC_mean_split$mean - dBIC_mean$mean)^2,na.rm = T)
# mean((dAR4_mean_split$mean - dAR4_mean$mean)^2,na.rm = T)
# 
# mean((dHP_mean_varyingLambda$mean - dHP_mean$mean)^2,na.rm = T)
# mean((dBIC_mean_varyingLambda$mean - dBIC_mean$mean)^2,na.rm = T)
# mean((dAR4_mean_varyingLambda$mean - dAR4_mean$mean)^2,na.rm = T)


load("date_recession.Rdata")
df.temp = recession.df[1,]
df.temp[1,] <- c("2020-03-01","2020-04-01")
recession.df <- rbind(recession.df,df.temp)
month(recession.df$start) <- month(recession.df$start) - 1
recession.df[26,2] <- recession.df[26,2] - 2
month(recession.df$end) <- month(recession.df$end) + 1
recession.df$end <- recession.df$end - 1
recession_1959.df = recession.df[recession.df$start>=min(df_BIC$DATE),]
recession_1959.df[1,] <- c("1960-06-01","1961-03-01")
recession_1959.df[2,] <- c("1969-12-01","1970-12-01")
recession_1959.df[3,] <- c("1973-12-01","1975-03-01")
recession_1959.df[4,] <- c("1980-03-01","1980-09-01")
recession_1959.df[5,] <- c("1981-09-01","1982-12-01")
recession_1959.df[6,] <- c("1990-09-01","1991-03-01")
recession_1959.df[7,] <- c("2001-03-01","2001-12-01")
recession_1959.df[8,] <- c("2007-12-01","2009-06-01")
recession_1959.df[9,] <- c("2019-12-01","2020-06-01")


theme1 = theme_bw( # base_family = "Times New Roman"
) +  theme(legend.title.align = 0.5, 
           plot.title = element_text(hjust = 0.5),
           legend.position="bottom", 
           text = element_text(size = 22),
           legend.title=element_text(size=22),
           panel.grid.major = element_blank(),
           panel.grid.minor = element_blank(),
           axis.text = element_text(size=22),
           axis.title = element_text(size=22),
           legend.text=element_text(size=22)
) 


glist <- list()
#### HP

HP_plot = t(as.matrix(df_HP[,2:length(var_names)]))
colnames(HP_plot) <- paste("date", seq(length(dates)), sep="")
rownames(HP_plot) <- paste("var", seq(length(var_names)-1), sep="")

df_HP_plot = as.data.frame(HP_plot)
df_HP_plot$var <- rownames(HP_plot)
mdf_HP_plot <- melt(df_HP_plot, id.vars = "var")
mdf_HP_plot$date <- as.numeric(gsub("date", "", mdf_HP_plot$variable))
mdf_HP_plot$date <- dates[mdf_HP_plot$date]



ggp = ggplot(mdf_HP_plot, aes(x=date, y=value, group = var)) +
  theme_bw() +
  theme(panel.grid=element_blank(), 
        #axis.text.y = element_blank(),
        #axis.ticks.y = element_blank(),
        legend.position = "none",
        axis.text=element_text(size=16 )) +
  geom_line(size=0.2, alpha=0.2, color = "darkseagreen3")  ## size, transparency and color of the lines

dHP_mean <- df_HP["DATE"]
dHP_mean$mean <- rowMeans(df_HP[,2:length(var_names)], na.rm = T)
ggp = ggp + geom_line(data = dHP_mean, aes(x=DATE, y=mean, group=NULL), 
                      size=0.6, alpha=1, color = "red")

print(ggp)
ggp = ggp + xlab("")+ ylab("")  + 
  scale_x_date(date_breaks = "5 years", date_minor_breaks = "1 year", 
               date_labels = "%Y", limits = as.Date(c("1959-01-01","2023-07-01")))
ggp = ggp + annotate("rect",xmin = as.Date(recession_1959.df$start, "%Y-%m-%d"),
                     xmax= as.Date(recession_1959.df$end, "%Y-%m-%d"),
                     ymin=-Inf,ymax=Inf, alpha=0.2, fill="black")  + theme1  + 
  theme(axis.text.x = element_blank(), 
        axis.ticks.x = element_blank(), 
        axis.title.x = element_blank()) + 
  scale_y_continuous(limits = c(-6,6),breaks = seq(-6,6,3))+
  theme(plot.background = element_rect(color = "white",
                                       size = 1),
        plot.margin = margin(t = 10,   
                             r = 20,   
                             b = -10,   
                             l = 5)) + xlab(" ") + ylab("HP")


print(ggp)


glist[[1]] <- ggp





# 
# #### 2HP
# 
# twoHP_plot = t(as.matrix(df_2HP[,2:length(var_names)]))
# colnames(twoHP_plot) <- paste("date", seq(length(dates)), sep="")
# rownames(twoHP_plot) <- paste("var", seq(length(var_names)-1), sep="")
# 
# df_2HP_plot = as.data.frame(twoHP_plot)
# df_2HP_plot$var <- rownames(twoHP_plot)
# mdf_2HP_plot <- melt(df_2HP_plot, id.vars = "var")
# mdf_2HP_plot$date <- as.numeric(gsub("date", "", mdf_2HP_plot$variable))
# mdf_2HP_plot$date <- dates[mdf_2HP_plot$date]
# 
# 
# 
# ggp = ggplot(mdf_2HP_plot, aes(x=date, y=value, group = var)) +
#   theme_bw() +
#   theme(panel.grid=element_blank(), 
#         #axis.text.y = element_blank(),
#         #axis.ticks.y = element_blank(),
#         legend.position = "none",
#         axis.text=element_text(size=16 )) +
#   geom_line(size=0.2, alpha=0.2, color = "darkseagreen3")  ## size, transparency and color of the lines
# 
# d2HP_mean <- df_2HP["DATE"]
# d2HP_mean$mean <- rowMeans(df_2HP[,2:length(var_names)], na.rm = T)
# ggp = ggp + geom_line(data = d2HP_mean, aes(x=DATE, y=mean, group=NULL), 
#                       size=0.6, alpha=1, color = "darkgreen")
# 
# print(ggp)
# ggp = ggp + xlab("")+ ylab("")  + 
#   scale_x_date(date_breaks = "5 years", date_minor_breaks = "1 year", 
#                date_labels = "%Y", limits = as.Date(c("1959-01-01","2023-07-01")))
# ggp = ggp + annotate("rect",xmin = as.Date(recession_1959.df$start, "%Y-%m-%d"),
#                      xmax= as.Date(recession_1959.df$end, "%Y-%m-%d"),
#                      ymin=-Inf,ymax=Inf, alpha=0.2, fill="black")  + theme1  + 
#   theme(axis.text.x = element_blank(), 
#         axis.ticks.x = element_blank(), 
#         axis.title.x = element_blank()) + 
#   scale_y_continuous(limits = c(-6,6),breaks = seq(-6,6,3))+
#   theme(plot.background = element_rect(color = "white",
#                                        size = 1),
#         plot.margin = margin(t = 10,   
#                              r = 20,   
#                              b = -10,   
#                              l = 5)) + xlab(" ") + ylab("2HP")
# 
# 
# print(ggp)
# 
# 
# glist[[2]] <- ggp


###### bHP 
BIC_plot = t(as.matrix(df_BIC[,2:length(var_names)]))
colnames(BIC_plot) <- paste("date", seq(length(dates)), sep="")
rownames(BIC_plot) <- paste("var", seq(length(var_names)-1), sep="")

df_BIC_plot = as.data.frame(BIC_plot)
df_BIC_plot$var <- rownames(BIC_plot)
mdf_BIC_plot <- melt(df_BIC_plot, id.vars = "var")
mdf_BIC_plot$date <- as.numeric(gsub("date", "", mdf_BIC_plot$variable))
mdf_BIC_plot$date <- dates[mdf_BIC_plot$date]


ggp = ggplot(mdf_BIC_plot, aes(x=date, y=value, group = var)) +
  theme_bw() +
  theme(panel.grid=element_blank(), 
        #axis.text.y = element_blank(),
        #axis.ticks.y = element_blank(),
        legend.position = "none",
        axis.text=element_text(size=16)) +
  geom_line(size=0.2, alpha=0.2, color = "darkseagreen3")  ## size, transparency and color of the lines

dBIC_mean <- df_BIC["DATE"]
dBIC_mean$mean <- rowMeans(df_BIC[,2:length(var_names)], na.rm = T)
ggp = ggp + geom_line(data = dBIC_mean, aes(x=DATE, y=mean, group=NULL), 
                      size=0.6, alpha=1, color = "blue")

print(ggp)
ggp = ggp + xlab("")+ ylab("") + 
  scale_x_date(date_breaks = "5 years", date_minor_breaks = "1 year", 
               date_labels = "%Y", limits = as.Date(c("1959-01-01","2023-07-01")))
ggp = ggp + annotate("rect",xmin = as.Date(recession_1959.df$start, "%Y-%m-%d"),
                     xmax= as.Date(recession_1959.df$end, "%Y-%m-%d"),
                     ymin=-Inf,ymax=Inf, alpha=0.2, fill="black")  + theme1  + 
  theme(axis.text.x = element_blank(), 
        axis.ticks.x = element_blank(), 
        axis.title.x = element_blank()) + 
  scale_y_continuous(limits = c(-6,6),breaks = seq(-6,6,3))+
  theme(plot.background = element_rect(color = "white",
                                       size = 1),
        plot.margin = margin(t = -20,   
                             r = 20,   
                             b = -5,   
                             l = 5)) + xlab(" ") + ylab("bHP")

print(ggp)

glist[[2]] <- ggp 
# cairo_pdf("graphs/BIC.pdf", width = 12, height = 6, family = "Times",
#           fallback_resolution = 1000)
# print(ggp)
# dev.off()

######## AR4 




AR4_plot = t(as.matrix(df_AR4[,2:length(var_names)]))
colnames(AR4_plot) <- paste("date", seq(length(dates)), sep="")
rownames(AR4_plot) <- paste("var", seq(length(var_names)-1), sep="")

df_AR4_plot = as.data.frame(AR4_plot)
df_AR4_plot$var <- rownames(AR4_plot)
mdf_AR4_plot <- melt(df_AR4_plot, id.vars = "var")
mdf_AR4_plot$date <- as.numeric(gsub("date", "", mdf_AR4_plot$variable))
mdf_AR4_plot$date <- dates[mdf_AR4_plot$date]

ggp = ggplot(mdf_AR4_plot, aes(x=date, y=value, group=var)) +
  theme_bw() +
  theme(panel.grid=element_blank(), 
        
        legend.position = "none",
        axis.text=element_text(size=16 )) +
  geom_line(size=0.2, alpha=0.2, color = "darkseagreen3") ## size, transparency and color of the lines

print(ggp)

dAR4_mean <- df_AR4["DATE"]
dAR4_mean$mean <- rowMeans(df_AR4[,2:length(var_names)], na.rm = T)
ggp = ggp + geom_line(data = dAR4_mean, aes(x=DATE, y=mean, group=NULL), 
                      size=0.6, alpha=1, color = "purple")
print(ggp)
ggp = ggp + xlab("")+ ylab("")  + 
  scale_x_date(date_breaks = "5 years", date_minor_breaks = "1 year", 
               date_labels = "%Y", limits = as.Date(c("1959-01-01","2023-07-01")))
ggp = ggp + annotate("rect",xmin = as.Date(recession_1959.df$start, "%Y-%m-%d"),
                     xmax= as.Date(recession_1959.df$end, "%Y-%m-%d"),
                     ymin=-Inf,ymax=Inf, alpha=0.2, fill="black") + theme1  + 
  scale_y_continuous(limits = c(-6,6),breaks = seq(-6,6,3))+
  theme(plot.background = element_rect(color = "white",
                                       size = 1),
        plot.margin = margin(t = -20,   
                             r = 20,   
                             b = -5,   
                             l = 5)) + xlab(" ") + ylab("HRF")


print(ggp)

glist[[3]] <- ggp
p.grid <- plot_grid(
  plotlist = glist, ncol = 1, align = "hv"
)
cairo_pdf("graphs/aggregate_quarter.pdf", width = 15, height = 16, family = "Times",
          fallback_resolution = 1000) 
print(p.grid )
dev.off()
# 


