
# compare <- function(x, trend, Title){
#   
#   x = as.ts(x)
#   n = length(x)
#   
#   if (Title == "Quarter"){
#     lam = 1600
#   }else if (Title == "Month"){
#     lam = 129600 
#   }else{
#     lam = 1.6e-5 * n^4 
#   }
#   
#   d0 = BoostedHP( x, lambda = lam, iter = FALSE)$trend
#   
#   d1 =  x - BoostedHP( x-d0 , lambda = lam, iter = FALSE)$cycle
#   
#   bhp.bic = BoostedHP( x, lambda=lam, iter = TRUE, stopping =  "BIC", Max_Iter = 200)
#   d2 = bhp.bic$trend
#   stop.bic = bhp.bic$iter_num
#   
#   if (Title == "Month"){
#     lag = 12
#   }else{
#     lag = 4
#   }
#   
#   reg = dynlm( x  ~ L(x, c(1:lag + 2*lag - 1) ) )
#   d3 = c( rep(NA,3*lag - 1), predict(reg) )
#   
#   
#   dd = cbind( x, d0, d1, d2, d3) 
#   
#   # trim_dd = dd[ (3*lag):(n),  ]
#   # trim_dd = dd[ 1:(n),  ]
#   
#   MSE = colMeans( ( dd[,2:5] - trend  )^2, na.rm = TRUE )
#   
#   return( list(MSE = MSE, 
#                # MSE.40 = colMeans( (trim_dd[ (n-47):(n-8),2:6] - trend[ (n-43):(n-4) ] )^2, na.rm = TRUE ), 
#                stop = stop.bic)   )
#   
# }


compare <- function(x, trend, Title){
  
  x = as.ts(x)
  n = length(x)
  
  if (Title == "Quarter"){
    lam = 1600
  }else if (Title == "Month"){
    lam = 129600 
  }else{
    lam = 1.6e-5 * n^4 
  }
  
  d0 = BoostedHP( x, lambda = lam, iter = FALSE)$trend
  
  d1 =  x - BoostedHP( x-d0 , lambda = lam, iter = FALSE)$cycle
  
  bhp.bic = BoostedHP( x, lambda=lam, iter = TRUE, stopping =  "BIC", Max_Iter = 200)
  d2 = bhp.bic$trend
  stop.bic = bhp.bic$iter_num
  
  if (Title == "Month"){
    lag = 12
  }else{
    lag = 4
  }
  
  reg = dynlm( x  ~ L(x, c(1:lag + 2*lag - 1) ) )
  d3 = c( rep(NA,3*lag - 1), predict(reg) )
  
  
  dd = cbind( x, d0, d1, d2, d3) 
  
  trim_dd = dd[ (3*lag):(n),  ]
  # trim_dd = dd[ 1:(n),  ]
  
  MSE = colMeans( (trim_dd[,2:5] - trend[ (3*lag):(n) ] )^2, na.rm = TRUE )
  
  return( list(MSE = MSE, 
               # MSE.40 = colMeans( (trim_dd[ (n-47):(n-8),2:6] - trend[ (n-43):(n-4) ] )^2, na.rm = TRUE ), 
               stop = stop.bic)   )
  
}



Simul_HP <- function(n, Title, Trend, Rep = 5000, sigmaI2 = 5, sigmaLUR = 1, rr = 0.5){
DD = matrix(NA, Rep, 80)
STOP = matrix(NA, Rep, 20)
pts1 <- Sys.time()

period = 8
if (Title == "Month"){
  period = 24
}

phi = 2*pi / period 

for (r in 1:Rep){
  print(r)
  noise = rnorm(n)
  
  if (Trend == "I(2)"){
    # The Arma Process 
    discard = 50
    alpha0 = .5

    b1 = 1
    b2 = 1/(1-4*cos(2*pi/period))
    
    nd = n+discard
    # sigma.u = sqrt(   sigmaI2^2 * (1+b2) * (1-b1-b2) * (1+b1-b2) / (1-b2)   ) 
    u =  sigmaI2 * rnorm( nd ) # the error term
    e = rep(0,nd )
    for (nn in 3:nd){
      e[nn] = b1*e[nn-1] + b2*e[nn-2] + u[nn]  # an AR process
    }
    e = e[ (discard + 1): nd]
    
    #### deterministic trends
    dett.lin = 100 * (1:(n/2)) / n 
    dett.cos =  5 * 100^(1/5) * ((1:n)/n)^( 1/5 ) * cos( 0.05*100^(0.9)*pi*((1:n)/n)^(.9) )
    dett.poly = 200 * ( (1:n ) / n )^3 
    dett.break = c( rep(0,n/2),  200 * ( ( (n/2+1):n ) / n )^3  )
    
    ### Stochastic trends
    x1 = cumsum(noise)
    x1 = cumsum(x1)
    trend1 = x1
    x1 = trend1 + e
    
    trend2 = trend1 + dett.poly
    x2 = trend2 + e
    
    trend3 = trend1 + dett.break
    x3 = trend3 + e
    
    trend4 = c( sigmaI2 *noise[1:(0.5*n)], cumsum(cumsum( noise[(n *0.5 + 1):n] ) )) + dett.break
    x4 = trend4 + e
    
    trend5 = trend4+dett.cos
    x5 = trend5 + e
    
    x.mat = cbind(x1,x2,x3,x4,x5)
    trend.mat = cbind(trend1,trend2,trend3,trend4,trend5)
    for (num in c(1:3)){
      out = compare(x.mat[,num], trend.mat[,num], Title)
      DD[r, (4*num-3):(4*num)] =  out$MSE
      STOP[r, num] = out$stop
    }
  
  }
  
  if (Trend == "LUR"){
  
  ## Local To Unity
 
  # The Arma Process for LUR, \sigma = 1
  discard = 50
  alpha0 = .5 

  b1 = 1
  b2 = 1/(1-4*cos(2*pi/period))
  
  nd = n+discard
  u =  sigmaLUR * rnorm( nd ) # the error term
  e = rep(0,nd )
  e[1:2] = u[1:2]
  for (nn in 3:nd){
    e[nn] = b1*e[nn-1] + b2*e[nn-2] + u[nn]  # an AR process
  }
  e = e[ (discard + 1): nd]

  
  #### deterministic trends
  dett.lin = 100 * (1:(n/2)) / n 
  dett.cos =  5 * 100^(1/5) * ((1:n)/n)^( 1/5 ) * cos( 0.05*100^(0.9)*pi*((1:n)/n)^(.9) )
  dett.poly = 200 * ( (1:n ) / n )^3 
  dett.break = c( rep(0,n/2),  200 * ( ( (n/2+1):n ) / n )^3  )
  
  ### Stochastic trends
  trend6 = DGP_LUR( n = n,  c = 3 )
  x6 = trend6 + e
  

  trend7 = trend6 + dett.poly
  x7 = trend7 + e
  
  trend8 = trend6 + dett.break
  x8 = trend8 + e
  
  trend9 = c( noise[1:(0.5*n)], trend6[1:(0.5*n)]) + dett.break 
  x9 = trend9 + e
  
  trend10 = trend9 + dett.cos 
  x10 = trend10 + e
  
  
  ################## LUR c = 0
  trend11 = DGP_LUR( n = n,  c = 0 )
  x11 = trend11 + e
  
  trend12 = trend11 + dett.poly  
  x12 = trend12 + e
  
  trend13 = trend11 + dett.break
  x13 = trend13 + e
  
  trend14 = c( noise[1:(0.5*n)], trend11[1:(0.5*n)]) + dett.break 
  x14 = trend14 + e
  
  trend15 = trend14+dett.cos 
  x15 = trend15 + e
  # 
  
  
  ############# LUR c = -3 
  trend16 = DGP_LUR( n = n,  c = -3 )
  x16 = trend16 + e  # 
  trend17 = trend16 + dett.poly
  x17 = trend17 + e
  
  trend18 = trend16 + dett.break
  x18 = trend18 + e
  
  
  trend19 = c( noise[1:(0.5*n)], trend16[1:(0.5*n)]) + dett.break 
  x19 = trend19 + e
  
  trend20 = trend19 + dett.cos 
  x20 = trend20 + e
  ######################
  
  x.mat = cbind(x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20)
  trend.mat = cbind(trend6,trend7,trend8,trend9,trend10,trend11,trend12,
                    trend13,trend14,trend15,trend16,trend17,trend18,trend19,trend20)
  
    for (num in c(6:8,11:13,16:18)){
      col = num - 5
      out = compare(x.mat[,col], trend.mat[,col], Title)
      DD[r, (4*num-3):(4*num)] =  out$MSE
      STOP[r, num] = out$stop
     }
  }
  
  }
  


D_mean = colMeans(DD)
D_mean = matrix(D_mean, ncol = 4, byrow = TRUE)
 
S_mean = colMeans(STOP)
S_mean = matrix(S_mean, ncol = 1, byrow = TRUE)
 
############# 
# cat("\n stopping time \n")
# print(S_mean)

save(DD, STOP, D_mean, S_mean, file = paste0(Title, "_", Trend, "_", n, "_Rep_", Rep, ".RData"))

pts2 = Sys.time() - pts1
print(pts2)

dta_export = cbind(D_mean, S_mean)
colnames(dta_export) <- c("MSE_HP","MSE_2HP","MSE_BIC","MSE_AR", "iter_BIC")

write.csv(dta_export,  file = paste0(Title, "_", Trend, "_", n, "_Rep_", Rep, ".csv"))
}